/* rolodump.c - dump Palm AddressBook archive (aba) format into my
 * personal rolodex file format.
 * 
 * This is a sample to demonstrate how to use the palm reader library
 */

#include <stdio.h>

#include "palmlib.h"

static void usage()
{
	fprintf(stderr, "rolodump file\n");
	exit(1);
}

static void dumpAddressEntry(AddressBookEntry *address, PalmRead *reader, FILE *out)
{
	if(address->categoryIndex <= 0) {
		// fprintf(out,"category: unknown");
		return;
	} else if(address->categoryIndex <= reader->getCategoryCount()) {
		if(strcmp(reader->getCategoryName(address->categoryIndex),
			"rolo") != 0) return;
	} else {
		return;
	}

#if NO_CARE
	if(title != NULL && *title != '\0') {
		fprintf(out, "title: %s\n", title);
	}
	if(address != NULL && *address != '\0') {
		fprintf(out, "%s\n", address);
	}
	if(city != NULL && *city != '\0') {
		fprintf(out, "%s", city);
	}
	if(state != NULL && *state != '\0') {
		fprintf(out, " %s", state);
	}
	if(zip != NULL && *zip != '\0') {
		fprintf(out, "  %s", zip);
	}
	if(country != NULL && *country != '\0') {
		fprintf(out, ", %s", country);
	}
	fprintf(out, "\n");
#endif

	int	emailIndex = -1;
	int	mobileIndex = -1;
	int	nPhones = 0;

	for(int pi = 0; pi < 5; pi += 1) {
		int li = address->phoneNumbers[pi]->labelIndex;
		if(*address->phoneNumbers[pi]->text == '\0') continue;

		if(li == 4) {
			emailIndex = pi;
		} else if (li == 7) {
			mobileIndex = pi;
		} else {
			nPhones += 1;
		}
	}

	for(int pi = 0; pi < 5; pi += 1) {
		int li = address->phoneNumbers[pi]->labelIndex;

		if(*address->phoneNumbers[pi]->text == '\0') continue;
		if(li == 4) continue;
		if(li == 7 && nPhones >= 1) continue;

		fprintf(out, "%s,", address->phoneNumbers[pi]->text);

		if(*address->firstname != '\0') {
			fprintf(out, "%s ", address->firstname);
		}
		if(*address->lastname != '\0') {
			fprintf(out, "%s", address->lastname);
		}
		if(*address->firstname == '\0' && *address->lastname == '\0') {
			if(address->company != NULL && *address->company != '\0') {
				fprintf(out, "%s", address->company);
			}
		}
		if(nPhones > 1) {
			if(0 <= li  && li < 10) {
				fprintf(out, " %s", PalmUtil::phoneTypes[li]);
			}
		}
		fprintf(out, ",");

		if(emailIndex >= 0) {
			fprintf(out, "%s",
				address->phoneNumbers[emailIndex]->text);
		}
		fprintf(out, ",");
		if(mobileIndex >= 0) {
			fprintf(out, "%s",
				address->phoneNumbers[mobileIndex]->text);
		}
		fprintf(out, "\n");
	}
}

static void dumpit(FILE *inp, FILE *out)
{
	PalmRead reader(inp);
	int	i;

	reader.loadheader();
	// reader.printheader(out);

	if(reader.getSignature() != PALM_ABA_SIG) {
		fprintf(stderr, "Not an address book file\n");
		return;
	}
	for(i = 0; i < reader.getNumEntries(); i++) {
		// printf("# %d\n", i);
		AddressBookEntry *address = new AddressBookEntry(&reader);
		dumpAddressEntry(address, &reader, out);
		delete address;
	}

	// long x = reader.getLong();
	// fprintf(out, "x = %d\n", x);
}


main(int argc, char *argv[])
{

	if(argc < 1) {
		usage();
		exit(1);
	}

	FILE *inp = fopen(argv[1], "r");
	if(inp == NULL) {
		fprintf(stderr, "Cannot open %s\n", argv[1]);
	}

	dumpit(inp, stdout);
}
