/* tclutil.h - tcl utility functions
 *
 */

/*
 * Copyright (c) 2002 Tony Aiuto
 *
 * Permission is granted to anyone to use this software for any purpose on
 * any computer system, and to alter it and redistribute it freely, subject
 * to the following restrictions:
 *
 * 1. Neither the authors nor their employers are responsible for
 *    any consequences of the use of this software.
 *
 * 2. The origin of this software must not be misrepresented, either by
 *    explicit claim or by omission.  Credit to the authors must appear
 *    in documentation and sources.
 *
 * 3. Altered versions must be plainly marked as such, and must not be
 *    misrepresented as being the original software.
 *
 * 4. This copyright notice may not be removed or altered.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL TONY AIUTO, HIS CURRENT OR PAST EMPLOYERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */


/*
 * Author: Tony Aiuto (tony@aiu.to)
 *
 * Edit History:
 * $Log: tclutil.h $
 * Revision tools_TonyAiuto/2 2002/09/07 05:48:48 TonyAiuto
 * 	add copyright stuff
 * 
 * Wed Sep  4 23:52:32 EDT 2002
 * tony - created
 */


class TCLutil {
public:
static int checkArgs(Tcl_Interp *interp, int argc, char *argv[], int expect);

};


/** TCLCommand is an interface which all added commands should obey
 * It wraps registration of the command with the interpreter
 *
 * All subclasses must implement command.
 * Subclasses which require client data should save that
 * in an internal member during their construction.  The
 * "command" method recieves the object pointer when invoked
 */

class TCLCommand {
public:
	TCLCommand(Tcl_Interp *, const char *commandname);
	virtual ~TCLCommand();

	/* the method called when the command is invoked.
	 * Note: clientData is always the object
	 */
	virtual int command(Tcl_Interp *interp, int argc, char *argv[]) = 0;

	// no public members

protected:

	// set the TCL result with a long
	void setResult(Tcl_Interp *, long);

	char	*name;

private:
	/* Internal dispatcher methods.  
	 * Note: clientData is always the object
	 */
	static void cleanupDispatch(ClientData);
	static int commandDispatch(ClientData clientData, Tcl_Interp *interp,
					int argc, char *argv[]);
};


/** Some utility macros */
#define  GETTER_I(name, member) \
	if (strcmp(argv[1], name) == 0) { \
		char	tmp[30]; \
		sprintf(tmp, "%d", member); \
		Tcl_SetResult(interp, tmp, TCL_VOLATILE); \
	}

#define  GETTER_S(name, member) \
	if (strcmp(argv[1], name) == 0) { \
		char *_tmp = (char *) member; \
		if(_tmp != NULL) { \
			Tcl_SetResult(interp, _tmp, TCL_VOLATILE); \
		} \
	}
#define  GETTER_W(name, member) \
	if (strcmp(argv[1], name) == 0) { \
		if(member == NULL) { \
			Tcl_SetResult(interp, "", TCL_STATIC); \
		} else { \
			tmpstr = ucs2_to_printable(member); \
			Tcl_SetResult(interp, tmpstr, TCL_DYNAMIC); \
		} \
	}
#define GETTER_8(name, member) \
	if (strcmp(argv[1], name) == 0) { \
		if(member == NULL) { \
			Tcl_SetResult(interp, "", TCL_STATIC); \
		} else { \
			tmpstr = ucs2_to_utf8(member); \
			Tcl_SetResult(interp, tmpstr, TCL_VOLATILE); \
			delete tmpstr; \
		} \
	}
